/*
     Copyright (C) 1996 - 1997  Masahito Yamaga (yamaga@ipc.chiba-u.ac.jp)

*/

#include "xdvi-config.h"
#include "xdvi.h"
#include "vfontmap.h"
#include "kpathsea/tex-file.h"
#ifdef	PTEX
#include "ft2.h"
#include <ctype.h>

#define NUM(a)	  ((int)(sizeof(a)/sizeof(*(a))))

static int vfontmaps_num = 0;
static int vfontmaps_max = 0;
static struct vfontmap *vfontmaps = NULL;

static void addfontmap(char *jfm, char *vf, int encoding)
{
    struct vfontmap *map;

    if (vfontmaps_num == vfontmaps_max) {
	if (vfontmaps_max == 0) vfontmaps_max = 8;
	else                    vfontmaps_max *= 2;
	vfontmaps = xrealloc(vfontmaps, sizeof(vfontmaps[0]) * vfontmaps_max);
    }
    map = &vfontmaps[vfontmaps_num];

    map->jfmname = xstrdup(jfm);
    map->vfname  = xstrdup(vf);
    map->encoding = encoding;
    map->face_index = 0;
    map->extend = 1.0;
    map->slant  = 0.0;
    map->variant = 0;
    map->ft2_index = -1;

    vfontmaps_num++;
}


const char *get_vfontmap_name(void)
{
    char *s = NULL;
    FILE *fp;
    static char *vfontmap = "vfontmap";
    static char *default_fontmap = DEFAULT_FONTMAP;

    if ((fp=fopen(vfontmap, "r")) != NULL) {
	fclose(fp);
	s = vfontmap;
    }
    if (s == NULL && strcmp(resource.vfontmap, default_fontmap) != 0) {
	s = resource.vfontmap;
    }
    if (s == NULL) s = getenv("VFONTMAP");
    if (s == NULL) s = kpse_find_file(vfontmap, kpse_fontmap_format, true);
    if (s == NULL) s = kpse_find_file(vfontmap, kpse_program_text_format, true);
    if (s == NULL) s = default_fontmap;
    return s;
}


int str2encoding(char *enc_name) {
    int i;
    static struct {
	char *enc_name;
	int encording;
    } encodes[] = {
	{ "JIS-H",      ENC_JIS_H },
	{ "JIS-V",      ENC_JIS_V },
	{ "Unicode-H",  ENC_UNICODE_H },
	{ "Unicode-V",  ENC_UNICODE_V },
	{ "Identity-H", ENC_IDENTITY },
	{ "Identity-V", ENC_IDENTITY },
	/* caution: Identity-H = Identity-V */
    };

    if (enc_name == NULL) return ENC_UNKNOWN;
    for (i=0; i<NUM(encodes); i++) {
	if (strcmp(enc_name, encodes[i].enc_name) == 0) {
	    return encodes[i].encording;
	}
    }
    return ENC_UNKNOWN;
}

void read_VFontmap(void)
{
    FILE *fp;
    const char *filename;
    char *s, *jfmname, *vfname, *encode;
    char line[BUFSIZ], line_orig[BUFSIZ], vfname2[BUFSIZ] = "";
    int encoding;

    filename = get_vfontmap_name();
    if ((fp = fopen(filename, "r")) == NULL) {
	fprintf(stderr,"Warning: Can't open \"%s\"\n", filename);
	fprintf(stderr,"         Default Values are Used.\n");
    } else {
	while (fgets(line, BUFSIZ, fp) != NULL) {
	    if ((s=strchr(line,'\n')) != NULL) *s = '\0'; /* break \n */
	    strcpy(line_orig, line);
	    if ((s=strchr(line,'%')) != NULL) *s = '\0';  /* break comment */
	    if ((s=strchr(line,'#')) != NULL) *s = '\0';  /* break comment */
	    encoding = 0;

	    if (*line == '/') {  /* font file for ft2 (full path) */
		strcpy(vfname2, strtok(line, " \t"));
		continue;
	    }
	    if (isspace(*line)) {	/* multi style */
		for (s=line; isspace(*s); s++) /**/;
		jfmname = strtok(s,    " \t");
		encode  = strtok(NULL, " \t");
		vfname  = vfname2;
	    } else {			/* single style */
		jfmname = strtok(line, " \t");
		encode  = strtok(NULL, " \t");
		vfname  = strtok(NULL, " \t");
	    }
	    encoding = str2encoding(encode);
	    if (jfmname == NULL) continue;
	    if (vfname == NULL) {
		fprintf(stderr,
			"No specified Vector Font name for \"%s\".\n", jfmname);
		continue;
	    }

	    if (encoding == ENC_UNKNOWN) {
		fprintf(stderr,
			"No specified encode for \"%s\"\n", jfmname);
		continue;
	    }

	    if (strtok(NULL, " \n\r") != NULL) {
		fprintf(stderr,
			"'%s', There is too much argument.\n", line_orig);
		continue;
	    }

	    if (globals.debug & DBG_PK) {
		fprintf(stderr, "JFM name is \"%s\".\n", jfmname);
		fprintf(stderr, "Font filename is \"%s\".\n", vfname);
		fprintf(stderr, "Font of FreeType is %s encode (#%d)\n",
			encode, encoding);
	    }
	    addfontmap(jfmname, vfname, encoding);
	}
	fclose(fp);
    }
}

struct vfontmap *getkanjifont(const char *fontname)
{
    int i;

    for (i=0; i<vfontmaps_num; i++) {
	if (strcmp(fontname, vfontmaps[i].jfmname) == 0) {
	    return &vfontmaps[i];
	}
    }
    return NULL;
}
#endif	/* PTEX */
